@@ -41,10 +41,16 @@ use Carp;
 use Scalar::Util qw{ blessed looks_like_number };
 
 use PPIx::Regexp;
+use PPIx::Regexp::Constant qw{
+    ARRAY_REF
+    @CARP_NOT
+};
 use PPIx::Regexp::Tokenizer;
 use PPIx::Regexp::Util qw{ __choose_tokenizer_class __instance };
 
-our $VERSION = '0.051';
+our $VERSION = '0.073';
+
+use constant LOCATION_WIDTH	=> 19;
 
 =head2 new
 
@@ -76,7 +82,7 @@ specified, it is passed through to
 L<< PPIx::Regexp->new()|PPIx::Regexp/new >>. It also causes an
 C<Encode::encode> to be done on any parse content dumped.
 
-=item explain boolean
+=item explain Boolean
 
 If true, this option causes the C<explain()> output of each object to be
 dumped.
@@ -96,24 +102,37 @@ hierarchy. This is ignored if the C<test> argument is true.
 
 The default is zero.
 
-=item ordinal boolean
+=item ordinal Boolean
 
 If true, this option causes the C<ordinal> values of
 L<PPIx::Regexp::Token::Literal|PPIx::Regexp::Token::Literal> objects to
 be dumped.
 
-=item perl_version boolean
+=item perl_version Boolean
 
 If true, this option causes the C<perl_version_introduced> and
 C<perl_version_removed> values associated with each object dumped to be
 displayed.
 
-=item postderef boolean
+=item postderef Boolean
+
+B<THIS ARGUMENT IS DEPRECATED>.  See L<DEPRECATION
+NOTICE|PPIx::QuoteLike/DEPRECATION NOTICE> in
+L<PPIx::QuoteLike|PPIx::QuoteLike> for the details.
 
 If true, postfix dereferences are recognized in code and interpolations.
 See the tokenizer's L<new()|PPIx::Regexp::Tokenizer/new> for details.
 
-=item strict boolean
+=item ppi Boolean
+
+If true, any Perl code contained in the object will be dumped.
+
+=item short Boolean
+
+If true, leading C<'PPIx::Regexp::'> will be removed from the class
+names in the output.
+
+=item strict Boolean
 
 This option is passed on to the parser, where it specifies whether the
 parse should assume C<use re 'strict'> is in effect.
@@ -125,13 +144,13 @@ parse produced when this option is asserted.
 
 The default is false.
 
-=item significant boolean
+=item significant Boolean
 
 If true, this option causes only significant elements to be dumped.
 
 The default is false.
 
-=item test boolean
+=item test Boolean
 
 If true, this option causes the output to be formatted as a regression
 test rather than as a straight dump. The output produced by asserting
@@ -141,7 +160,7 @@ kind.
 
 The default is false.
 
-=item tokens boolean
+=item tokens Boolean
 
 If true, this option causes a dump of tokenizer output rather than of a
 full parse of the regular expression. This is forced true if the dump is
@@ -178,9 +197,12 @@ ignored.
     my %default = (
 	explain	=> 0,
 	indent	=> 2,
+	locations	=> 0,
 	margin	=> 0,
 	ordinal	=> 0,
 	perl_version => 0,
+	ppi	=> 0,
+	short	=> 0,
 	significant => 0,
 	test	=> 0,
 	tokens	=> 0,
@@ -217,17 +239,21 @@ ignored.
 	    $self->{tokens} = 1;
 	} elsif ( __instance( $re, 'PPIx::Regexp::Element' ) ) {
 	    $self->{object} = $re;
-	} elsif ( ref $re eq 'ARRAY' ) {
+	} elsif ( ARRAY_REF eq ref $re ) {
 	    $self->{object} = $re;
 	} elsif ( ref $re && ! __instance( $re, 'PPI::Element' ) ) {
 	    croak "Do not know how to dump ", ref $re;
 	} elsif ( $self->{tokens} ) {
 	    my $tokenizer_class = __choose_tokenizer_class( $re, \%args )
 		or croak 'Unsupported data type';
+	    __instance( $re, 'PPI::Element' )
+		or $args{location} = [ 1, 1, 1, 1, undef ];
 	    $self->{object} =
 		$tokenizer_class->new( $re, %args )
 		    or Carp::croak( $tokenizer_class->errstr() );
 	} else {
+	    __instance( $re, 'PPI::Element' )
+		or $args{location} = [ 1, 1, 1, 1, undef ];
 	    $self->{object} =
 		PPIx::Regexp->new( $re, %args )
 		    or Carp::croak( PPIx::Regexp->errstr() );
@@ -254,7 +280,7 @@ sub list {
     my ( $self ) = @_;
     my $lister = $self->{test} ? '__PPIX_DUMPER__test' : '__PPIX_DUMPER__dump';
 
-    ref $self->{object} eq 'ARRAY'
+    ARRAY_REF eq ref $self->{object}
 	and return ( map { $_->$lister( $self ) } @{ $self->{object} } );
 
     return $self->{object}->$lister( $self );
@@ -270,6 +296,13 @@ This method simply prints the result of L</string> to standard out.
 
 sub print : method {	## no critic (ProhibitBuiltinHomonyms)
     my ( $self ) = @_;
+    # Non-characters and Non-Unicode code points are explicitly allowed
+    # as delimiters, at least as of 5.29.0, which is where unassigned
+    # and combining code points became illegal. Unfortunately the
+    # warnings below were not introduced until 5.14, so have to go for
+    # the next-higher warning category.
+    # no warnings qw{ nonchar non_unicode };	## no critic (ProhibitNoWarnings)
+    no warnings qw{ utf8 };	## no critic (ProhibitNoWarnings)
     print $self->string();
     return;
 }
@@ -300,7 +333,7 @@ sub _safe {
 	}
 	if ( ! defined $item ) {
 	    push @rslt, 'undef';
-	} elsif ( ref $item eq 'ARRAY' ) {
+	} elsif ( ARRAY_REF eq ref $item ) {
 	    push @rslt, join( ' ', '[', $self->_safe( @{ $item } ), ']' );
 	} elsif ( looks_like_number( $item ) ) {
 	    push @rslt, $item;
@@ -323,18 +356,40 @@ sub __nav {
     my $rslt = $self->_safe( @args );
     $rslt =~ s/ ' (\w+) ' , /$1 =>/smxg;
     $rslt =~ s/ \[ \s+ \] /[]/smxg;
-    $rslt =~ s/ \[ \s* ( \d+ ) \s* \] /$1/smxg;
+    $rslt =~ s/ \[ \s* ( [0-9]+ ) \s* \] /$1/smxg;
     return $rslt;
 }
 
 sub _perl_version {
     my ( undef, $elem ) = @_;		# Invocant unused
 
-    my $rslt = $elem->perl_version_introduced() . ' <= $]';
-    if ( my $max = $elem->perl_version_removed() ) {
-	$rslt .= ' < ' . $max;
-    }
-    return $rslt;
+    return $elem->requirements_for_perl();
+}
+
+sub _ppi {
+    my ( $self, $elem ) = @_;
+
+    $self->{ppi}
+	and $elem->can( 'ppi' )
+	or return;
+
+    require PPI::Dumper;
+
+    # PPI::Dumper reports line_number(), but I want
+    # logical_line_number(). There is no configuration for this, but the
+    # interface is public, so I mung it to do what I want.
+    my $locn = PPI::Element->can( 'location' );
+    local *PPI::Element::location = sub {
+	my $loc = $locn->( @_ );
+	$loc->[0] = $loc->[3];
+	return $loc;
+    };
+
+    my $dumper = PPI::Dumper->new( $elem->ppi(),
+	map { $_ => $self->{$_} } qw{ indent locations },
+    );
+
+    return $dumper->list();
 }
 
 sub _content {
@@ -342,7 +397,7 @@ sub _content {
     defined $dflt or $dflt = '';
 
     defined $elem or return $dflt;
-    if ( ref $elem eq 'ARRAY' ) {
+    if ( ARRAY_REF eq ref $elem ) {
 	my $rslt = join '',
 	    map { $self->_content( $_ ) }
 	    grep { ! $self->{significant} || $_->significant() }
@@ -354,14 +409,14 @@ sub _content {
 }
 
 sub _tokens_dump {
-    my ( $self, $elem ) = @_;
+    my ( $self, $elem, $depth ) = @_;
 
     not $self->{significant} or $elem->significant() or return;
 
     my @rslt;
     foreach my $token ( $elem->tokens() ) {
 	not $self->{significant} or $token->significant() or next;
-	push @rslt, $token->__PPIX_DUMPER__dump( $self );
+	push @rslt, $token->__PPIX_DUMPER__dump( $self, $depth );
     }
     return @rslt;
 }
@@ -372,7 +427,7 @@ sub _format_default_modifiers {
     foreach my $attr ( qw{ default_modifiers parse postderef strict } ) {
 	defined ( my $val = $self->{$attr} )
 	    or next;
-	'ARRAY' eq ref $val
+	ARRAY_REF eq ref $val
 	    and not @{ $val }
 	    and next;
 	push @arg, "$attr => @{[ $self->_safe( $val ) ]}";
@@ -380,6 +435,13 @@ sub _format_default_modifiers {
     return sprintf '%-8s( %s );', $subr, join ', ', @arg;
 }
 
+sub _format_matcher_dump {
+    my ( undef, $elem ) = @_;
+    my $value = $elem->is_matcher();
+    return sprintf 'is_matcher=%s',
+	$value ? 'true' : defined $value ?  'false' : 'undef';
+}
+
 sub _format_modifiers_dump {
     my ( undef, $elem ) = @_;		# Invocant unused
     my %mods = $elem->modifiers();
@@ -455,20 +517,29 @@ sub PPIx::Regexp::__PPIX_DUMPER__test {
 }
 
 sub PPIx::Regexp::Node::__PPIX_DUMPER__dump {
-    my ( $self, $dumper ) = @_;
+    my ( $self, $dumper, $depth ) = @_;
+
+    $depth ||= 0;
 
     $dumper->{tokens}
-	and return $dumper->_tokens_dump( $self );
+	and return $dumper->_tokens_dump( $self, $depth );
 
     not $dumper->{significant} or $self->significant() or return;
 
-    my @rslt = ref $self;
+    my @rslt = ( ref $self );
+    $dumper->{short}
+	and $rslt[0] =~ s/ \A PPIx::Regexp:: //smx;
     $self->isa( 'PPIx::Regexp' )
 	and $rslt[-1] .= $dumper->{verbose}
 	    ? sprintf "\tfailures=%d\tmax_capture_number=%d",
 		$self->failures(), $self->max_capture_number()
 	    : sprintf "\tfailures=%d", $self->failures();
 
+    substr $rslt[0], 0, 0, ' ' x ( $depth * $dumper->{indent} );
+
+    $dumper->{locations}
+	and substr $rslt[0], 0, 0, ' ' x LOCATION_WIDTH;
+
     $dumper->{perl_version}
 	and $rslt[-1] .= "\t" . $dumper->_perl_version( $self );
 
@@ -480,9 +551,9 @@ sub PPIx::Regexp::Node::__PPIX_DUMPER__dump {
 		$dumper, pop @rslt );
     }
 
-    my $indent = ' ' x $dumper->{indent};
+    $depth++;
     foreach my $elem ( $self->children() ) {
-	push @rslt, map { $indent . $_ } $elem->__PPIX_DUMPER__dump( $dumper );
+	push @rslt, $elem->__PPIX_DUMPER__dump( $dumper, $depth );
     }
     return @rslt;
 }
@@ -495,7 +566,7 @@ sub PPIx::Regexp::Node::__PPIX_DUMPER__test {
     my @rslt;
     @rslt = (
 	'choose  ( ' . $dumper->__nav( $self->nav() ) . ' );',
-	'class   ( ' . $dumper->_safe( ref $self ) . ' );',
+	'klass   ( ' . $dumper->_safe( ref $self ) . ' );',
 	'count   ( ' . scalar $self->children() . ' );',
     );
 
@@ -540,7 +611,9 @@ sub _format_value {
     );
 
     sub PPIx::Regexp::Structure::__PPIX_DUMPER__dump {
-	my ( $self, $dumper ) = @_;
+	my ( $self, $dumper, $depth ) = @_;
+
+	$depth ||= 0;
 
 	not $dumper->{significant} or $self->significant() or return;
 
@@ -550,6 +623,13 @@ sub _format_value {
 	    push @delim, @elem ? $dumper->_content( \@elem ) : $dflt{$method};
 	}
 	my @rslt = ( ref $self, "$delim[0]$delim[1] ... $delim[2]" );
+	$dumper->{short}
+	    and $rslt[0] =~ s/ \A PPIx::Regexp:: //smx;
+
+	substr $rslt[0], 0, 0, ' ' x ( $depth * $dumper->{indent} );
+
+	$dumper->{locations}
+	    and substr $rslt[0], 0, 0, ' ' x LOCATION_WIDTH;
 
 	$dumper->{perl_version}
 	    and push @rslt, $dumper->_perl_version( $self );
@@ -567,6 +647,8 @@ sub _format_value {
 	    $self->isa( 'PPIx::Regexp::Structure::Modifier' )
 		and push @rslt, $dumper->_format_modifiers_dump(
 		$self->type( 0 ) );
+
+	    push @rslt, $dumper->_format_matcher_dump( $self );
 	}
 
 	foreach my $method ( 'start', undef, 'finish' ) {
@@ -584,10 +666,9 @@ sub _format_value {
 	    and push @rslt, $self->__PPIX_DUMPER__dump_explanation(
 		$dumper, pop @rslt );
 
-	my $indent = ' ' x $dumper->{indent};
+	$depth++;
 	foreach my $elem ( $self->children() ) {
-	    push @rslt, map { $indent . $_ }
-		$elem->__PPIX_DUMPER__dump( $dumper );
+	    push @rslt, $elem->__PPIX_DUMPER__dump( $dumper, $depth );
 	}
 	return @rslt;
     }
@@ -602,7 +683,7 @@ sub PPIx::Regexp::Structure::__PPIX_DUMPER__test {
     my @nav = $self->nav();
     my @rslt = (
 	'choose  ( ' . $dumper->__nav( @nav ) . ' );',
-	'class   ( ' . $dumper->_safe( ref $self ) . ' );',
+	'klass   ( ' . $dumper->_safe( ref $self ) . ' );',
 	'count   ( ' . scalar $self->children() . ' );',
     );
     if ( $dumper->{verbose} ) {
@@ -621,7 +702,7 @@ sub PPIx::Regexp::Structure::__PPIX_DUMPER__test {
 	    my $elem = $eles[$inx];
 	    push @rslt, 'choose  ( ' . $dumper->__nav(
 		@nav, $method, $inx ) . ' );',
-		'class   ( ' . $dumper->_safe( ref $elem || $elem ) . ' );',
+		'klass   ( ' . $dumper->_safe( ref $elem || $elem ) . ' );',
 		'content ( ' . $dumper->_safe( $elem ) . ' );';
 	}
     }
@@ -632,9 +713,11 @@ sub PPIx::Regexp::Structure::__PPIX_DUMPER__test {
 }
 
 sub PPIx::Regexp::Tokenizer::__PPIX_DUMPER__dump {
-    my ( $self, $dumper ) = @_;
+    my ( $self, $dumper, $depth ) = @_;
 
-    return $dumper->_tokens_dump( $self );
+    $depth ||= 0;
+
+    return $dumper->_tokens_dump( $self, $depth );
 
 }
 
@@ -645,13 +728,38 @@ sub PPIx::Regexp::Tokenizer::__PPIX_DUMPER__test {
 }
 
 sub PPIx::Regexp::Token::__PPIX_DUMPER__dump {
-    my ( $self, $dumper ) = @_;
+    my ( $self, $dumper, $depth ) = @_;
+
+    $depth ||= 0;
 
     not $dumper->{significant}
 	or $self->significant()
 	or return;
 
     my @rslt = ( ref $self, $dumper->_safe( $self ) );
+    $dumper->{short}
+	and $rslt[0] =~ s/ \A PPIx::Regexp:: //smx;
+
+    substr $rslt[0], 0, 0, ' ' x ( $depth * $dumper->{indent} );
+
+    $dumper->{locations}
+	and substr $rslt[0], 0, 0,
+		sprintf '[ % 4d, % 3d, % 3d ] ',
+		    $self->logical_line_number(),
+		    $self->column_number(),
+		    $self->visual_column_number();
+
+    my @ppi;
+    @ppi = $dumper->_ppi( $self )
+	and shift @ppi;	# Ignore PPI::Document
+    foreach ( @ppi ) {
+	if ( $dumper->{locations} ) {
+	    s/ ( [0-9]+ \s+ \] ) /$1  /smxg
+		or substr $_, 0, 0, '  ';
+	} else {
+	    substr $_, 0, 0, '  ';
+	}
+    }
 
     $dumper->{perl_version}
 	and push @rslt, $dumper->_perl_version( $self );
@@ -693,6 +801,7 @@ sub PPIx::Regexp::Token::__PPIX_DUMPER__dump {
 		push @rslt, $dumper->_format_modifiers_dump( $self );
 	    }
 
+	    push @rslt, $dumper->_format_matcher_dump( $self );
 	}
 
 	@rslt = ( join "\t", @rslt );
@@ -701,6 +810,8 @@ sub PPIx::Regexp::Token::__PPIX_DUMPER__dump {
 	    and push @rslt, $self->__PPIX_DUMPER__dump_explanation(
 		$dumper, pop @rslt );
 
+	push @rslt, @ppi;
+
 	return @rslt;
     }
 }
@@ -713,7 +824,7 @@ sub PPIx::Regexp::Token::__PPIX_DUMPER__test {
     @nav or @nav = $self->nav();
     my @rslt = (
 	'choose  ( ' . join(', ', $dumper->__nav( @nav ) ) . ' );',
-	'class   ( ' . $dumper->_safe( ref $self ) . ' );',
+	'klass   ( ' . $dumper->_safe( ref $self ) . ' );',
 	'content ( ' . $dumper->_safe( $self ) . ' );',
     );
 
@@ -762,7 +873,7 @@ __END__
 =head1 SUPPORT
 
 Support is by the author. Please file bug reports at
-L<http://rt.cpan.org>, or in electronic mail to the author.
+L<https://rt.cpan.org>, or in electronic mail to the author.
 
 =head1 AUTHOR
 
@@ -770,7 +881,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009-2017 by Thomas R. Wyant, III
+Copyright (C) 2009-2020 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
